﻿using System;
using System.Linq;
using System.Windows.Forms;
using System.IO;
using System.ServiceModel;
using System.Xml;
using System.Xml.Schema;

using Sycamores.MobileShare.Client.Communications;
using Res = Sycamores.MobileShare.Client.Properties.Resources;

namespace Sycamores.MobileShare.Client
{
    public partial class MainForm : Form
    {
        #region Menu Enumerations

        private enum BaseMenu : int
        {
            Left = 0,
            Right,
        }
        private enum RightMenu : int
        {
            Edit = 0,
            View ,
            Sort,
            Option,
            Refresh,
            Properties,
            Search,
            About,
            Exit,
        }
        private enum ViewMenu : int
        {
            LgIcon = 0,
            SmIcon,
            List,
            Detail,
        }
        private enum SortMenu : int
        {
            Name = 0,
            Date,
            Size,
            Type,
        }

        #endregion

        #region Properties

        private IEntranceEx mService;
        private ApplicationDataFile mApplicationDataFile;
        private ApplicationDataElement mAppData;

        private string mDomain;
        private string mPath;
        private uint mLevel;

        private IEntranceEx GetService(Uri address)
        {
            return new EntranceClient(EntranceClient.CreateDefaultBinding(), new EndpointAddress(address));
        }

        private IEntranceEx Service
        {
            get
            {
                if (mService == null)
                    mService = GetService(mAppData.Servers[mAppData.TargetServer].ToUri());

                return mService;
            }
        }

        #endregion

        /// <summary>Default constructor.</summary>
        public MainForm()
        {
            InitializeComponent();

            try
            {
                mApplicationDataFile = ApplicationDataFile.GetFile();
                mAppData = mApplicationDataFile.ApplicationDataElement;
            }
            catch (XmlException ex) { HandleXmlExceptions(ex); }
            catch (XmlSchemaValidationException ex) { HandleXmlExceptions(ex); }
        }

        private void HandleXmlExceptions(Exception ex)
        {
            System.Diagnostics.Debug.WriteLine(ex.Message);

            MessageBox.Show(Res.ErrorAppDataCorrupt
                , Res.CaptionError
                , MessageBoxButtons.OK
                , MessageBoxIcon.Asterisk
                , MessageBoxDefaultButton.Button1);

            ApplicationDataFile.Purge();

            MenuExitItem_Click(this, EventArgs.Empty);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            this.Activate();
            this.BringToFront();

            CheckServerConfiguration();

            PopulateList(string.Empty, string.Empty, 0);
        }

        private void CheckServerConfiguration()
        {
            if (mAppData.TargetServer.Length <= 0)
            {
                MessageBox.Show(Res.ErrorNoServers
                              , Res.CaptionWelcome
                              , MessageBoxButtons.OK
                              , MessageBoxIcon.Asterisk
                              , MessageBoxDefaultButton.Button1);

                ServerMenuForm.ShowEditDialog(mAppData, mAppData.Servers.GetNewServer());

                mApplicationDataFile.Save();
            }
        }

        #region Populate List

        private void PopulateList(string domain, string path, uint level)
        {
            PrePopulate();

            try
            {
                switch (level)
                {
                    case 0:
                        PopulateNetworkListItems();
                        break;
                    case 1:
                        PopulateDomainListItems(domain);
                        break;
                    case 2:
                        PopulateComputerListItems(path);
                        break;
                    case 3:
                        PopulateShareListItems(path);
                        break;
                    case 4:
                        PopulateRootListItems(path);
                        break;
                    default:
                        PopulateFolderListItems(path, level);
                        break;
                }
            }
            catch (Microsoft.Tools.ServiceModel.CFFaultException ex) { HandleCFException(ex.FaultMessage); }
            catch (EndpointNotFoundException ex) { HandleEndpointException(ex); }
            catch (TimeoutException ex) { System.Diagnostics.Debug.WriteLine(ex.Message); }

            PostPopulate();
        }

        private void PopulateNetworkListItems()
        {
            foreach (var item in Service.GetDomains())
                BListView.AddListViewItemDomain(item);

            BListView.Text = Res.TitleNetwork;

            mDomain = string.Empty;
            mPath = string.Empty;
            mLevel = 0;

            EditMenuItem.Enabled = false;
            PropertiesMenuItem.Enabled = false;
            SearchMenuItem.Enabled = false;
        }

        private void PopulateDomainListItems(string domain)
        {
            foreach (var item in Service.GetComputers(domain))
                BListView.AddListViewItemComputer(item);

            BListView.Text = domain;

            mDomain = domain;
            mPath = string.Empty;
            mLevel = 1;

            EditMenuItem.Enabled = false;
            PropertiesMenuItem.Enabled = true;
            SearchMenuItem.Enabled = false;
        }

        private void PopulateComputerListItems(string computer)
        {
            char backslash = '\\';

            computer = computer.TrimStart(backslash);

            if (computer.Contains(backslash))
                computer = computer.Substring(0, computer.IndexOf(backslash));

            foreach (var item in Service.GetShares(computer))
                BListView.AddListViewItemShares(item);

            BListView.Text = string.Concat(string.Concat("\\\\", computer));

            mPath = BListView.Text;
            mLevel = 2;

            EditMenuItem.Enabled = false;
            PropertiesMenuItem.Enabled = true;
            SearchMenuItem.Enabled = false;
        }

        private void PopulateShareListItems(string share)
        {
            PopulateFolderListItems(share, 3);
        }

        private void PopulateRootListItems(string root)
        {
            PopulateFolderListItems(root, 4);
        }

        private void PopulateFolderListItems(string folder, uint level)
        {
            foreach (DirectoryInfo item in from dir in Service.GetDirectories(folder)
                                           where dir != null
                                           && CheckHidden(dir.Attributes)
                                           && CheckSystem(dir.Attributes)
                                           select dir)
            {
                BListView.AddListViewItemDirectory(item);
            }

            foreach (FileInfo item in from file in Service.GetFiles(folder)
                                      where file != null
                                      && CheckHidden(file.Attributes)
                                      && CheckSystem(file.Attributes)
                                      select file)
            {
                BListView.AddListViewItemFile(item);
            }

            BListView.Text = Path.Combine(BListView.Text, folder);

            mPath = folder;
            mLevel = level;

            if (BListView.ListView.SelectedIndices.Count > 0) EditMenuItem.Enabled = true;
            PropertiesMenuItem.Enabled = true;
            SearchMenuItem.Enabled = true;
        }

        private bool CheckSystem(int attributes)
        {
            bool showsystem = !mAppData.HideSystemFiles;
            bool issystem = (((System.IO.FileAttributes)attributes) & (FileAttributes.System)).Equals(FileAttributes.System);

            if (showsystem || !issystem)
                return true;
            else
                return false;
        }

        private bool CheckHidden(int attributes)
        {
            bool showhidden = mAppData.ShowHiddenFiles;
            bool ishidden = (((System.IO.FileAttributes)attributes) & (FileAttributes.Hidden)).Equals(FileAttributes.Hidden);

            if (showhidden || !ishidden)
                return true;
            else
                return false;
        }

        private void PrePopulate()
        {
            Cursor.Current = Cursors.WaitCursor;
            MenuUpItem.Text = Res.MenuCancel;
            MenuMenuItem.Enabled = false;
            MenuMenuItem.Text = string.Empty;
            BListView.ListView.Items.Clear();
        }

        private void PostPopulate()
        {
            Cursor.Current = Cursors.Default;
            MenuUpItem.Enabled = mLevel != 0;
            MenuUpItem.Text = Res.MenuUp;
            MenuMenuItem.Enabled = true;
            MenuMenuItem.Text = Res.MenuMenu;

            if (BListView.ListView.Items.Count > 0)
            {
                BListView.ListView.Items[0].Selected = true;
                BListView.ListView.Items[0].Focused = true;
            }
        }

        #endregion

        #region Menu Event Handlers

        private void ViewListItem_Click(object sender, EventArgs e)
        {
            ResetViewItemChecks(View.List, ViewMenu.List);
        }

        private void ViewDetailItem_Click(object sender, EventArgs e)
        {
            ResetViewItemChecks(View.Details, ViewMenu.Detail);
        }

        private void ViewSmIconItem_Click(object sender, EventArgs e)
        {
            ResetViewItemChecks(View.SmallIcon, ViewMenu.SmIcon);
        }

        private void ViewLgIconsItem_Click(object sender, EventArgs e)
        {
            ResetViewItemChecks(View.LargeIcon, ViewMenu.LgIcon);
        }

        private void ResetViewItemChecks(View type, ViewMenu menuItem)
        {
            BListView.ListView.View = type;

            foreach (MenuItem item in MainMenu.MenuItems[(int)BaseMenu.Right].MenuItems[(int)RightMenu.View].MenuItems)
            {
                item.Checked = false;
            }

            MainMenu.MenuItems[(int)BaseMenu.Right].MenuItems[(int)RightMenu.View].MenuItems[(int)menuItem].Checked = true;
        }

        private void MenuExitItem_Click(object sender, EventArgs e)
        {
            if (mApplicationDataFile != null) mApplicationDataFile.Save();
            this.Close();
        }

        private void PropertiesMenuItem_Click(object sender, EventArgs e)
        {
            if (BListView.ListView.SelectedIndices.Count > 0)
            {
                int index = BListView.ListView.SelectedIndices[0];
                object item = BListView.ListView.Items[index].Tag;

                try
                {
                    DirectoryInfo directory = item as DirectoryInfo;

                    if (directory != null)
                    {
                        using (DirectoryPropertiesForm form = new DirectoryPropertiesForm(directory))
                            form.ShowDialog();
                    }

                    FileInfo file = item as FileInfo;

                    if (file != null)
                    {
                        using (FilePropertiesForm form = new FilePropertiesForm(file))
                            form.ShowDialog();
                    }
                }
                catch (CommunicationException ex)
                {
                    HandleCommunicationsException(ex.Message);
                }

            }
        }

        #endregion

        #region Navigation

        private void RefreshMenuItem_Click(object sender, EventArgs e) { PopulateList(mDomain, mPath, mLevel); }

        private void MenuOptions_Click(object sender, EventArgs e)
        {
            string oldserver = mAppData.TargetServer;

            using (OptionsMenuForm optionsform = new OptionsMenuForm(mAppData))
            {
                switch (optionsform.ShowDialog())
                {
                    case DialogResult.OK:
                        mApplicationDataFile.Save();

                        if (!mAppData.Servers[mAppData.TargetServer].Equals(mAppData.Servers[oldserver]))
                        {
                            mService = GetService(mAppData.Servers[mAppData.TargetServer].ToUri());

                            PopulateList(string.Empty, string.Empty, 0);
                        }

                        break;
                    case DialogResult.Ignore:
                        mApplicationDataFile.Save();
                        break;
                }
            }
        }

        private void BListView_ItemActivate(object sender, EventArgs e)
        {
            if (sender == null) return;

            ListView list = (ListView)sender;

            if (list.SelectedIndices.Count <= 0) return;
            if (list.Items[list.SelectedIndices[0]].Tag == null) return;

            string itemtag = list.Items[list.SelectedIndices[0]].Tag.ToString();
            string domain = string.Empty;
            uint level = mLevel + 1;
            DirectoryInfo info = null;

            switch (mLevel)
            {
                case 0:
                    domain = itemtag;
                    break;
                case 1:
                    break;
                case 2:
                    if (itemtag.Contains("print$"))
                    {
                        NormanCoordinate();
                        return;
                    }
                    else
                        itemtag = Path.Combine(mPath, itemtag);
                    break;
                default:
                    info = list.Items[list.SelectedIndices[0]].Tag as DirectoryInfo;

                    if (info == null)
                    {
                        NormanCoordinate();
                        return;
                    }

                    itemtag = info.FullName;

                    break;
            }

            PopulateList(domain, itemtag, level);
        }

        private void MenuUpItem_Click(object sender, EventArgs e)
        {
            uint level = mLevel - 1;
            string parent;

            switch (level)
            {
                case 0:
                    PopulateList(string.Empty, string.Empty, level);
                    break;
                case 1:
                    PopulateList(mDomain, string.Empty, level);
                    break;
                default:
                    DirectoryInfo info = Service.GetDirectoryInfo(mPath);

                    parent = info.Parent;

                    if (string.IsNullOrEmpty(parent))
                        parent = info.Root;

                    PopulateList(mDomain, parent, level);
                    break;
            }
        }

        #endregion

        private void NormanCoordinate()
        {
            MessageBox.Show(Res.NormanCoordinate
                                     , Res.CaptionNotImpl
                                     , MessageBoxButtons.OK
                                     , MessageBoxIcon.Exclamation
                                     , MessageBoxDefaultButton.Button1);
        }

        private void HandleCommunicationsException(string message)
        {
            System.Diagnostics.Debug.WriteLine(message);
        }

        private void HandleCFException(string message)
        {
            System.Diagnostics.Debug.WriteLine(message);

            System.Xml.Linq.XName fault = System.Xml.Linq.XName.Get("Fault", "http://schemas.xmlsoap.org/soap/envelope/");
            System.Xml.Linq.XElement faultmessage = System.Xml.Linq.XElement.Parse(message);

            System.Diagnostics.Debug.WriteLine(faultmessage.Elements("faultstring").FirstOrDefault().Value);

            MessageBox.Show(faultmessage.Elements("faultstring").FirstOrDefault().Value
                , Res.CaptionMobileShare
                , MessageBoxButtons.OK
                , MessageBoxIcon.Hand
                , MessageBoxDefaultButton.Button1);

            RefreshMenuItem_Click(this, EventArgs.Empty);
        }

        private void HandleEndpointException(EndpointNotFoundException ex)
        {
            MessageBox.Show(string.Format(Res.ErrorNoServer, mAppData.GetTargetServer().ToString())
               , ex.InnerException.Message
               , MessageBoxButtons.OK
               , MessageBoxIcon.Exclamation
               , MessageBoxDefaultButton.Button1);
            System.Diagnostics.Debug.WriteLine(ex.Message);

            BListView.SetRedText(Res.NoServer);
        }

        #region Edit Menu

        private void EditDeleteMenuItem_Click(object sender, EventArgs e)
        {
            if (BListView.ListView.SelectedIndices.Count > 0)
            {
                ListView list = BListView.ListView;
                ListViewItem item = list.Items[list.SelectedIndices[0]];
                DirectoryInfo info = item.Tag as DirectoryInfo;

                if (info != null)
                {
                    DialogResult result = MessageBox.Show(string.Format(Res.DialogConfirmDeleteDirectory, info.Name)
                      , Res.CaptionConfirmFolderDelete
                      , MessageBoxButtons.YesNo
                      , MessageBoxIcon.Question
                      , MessageBoxDefaultButton.Button2);

                    if (result.Equals(DialogResult.Yes))
                    {
                        try
                        {
                            Service.Delete(info.FullName, true);
                            BListView.ListView.Items.Remove(item);
                        }
                        catch (Microsoft.Tools.ServiceModel.CFFaultException ex) { HandleCFException(ex.FaultMessage); }
                        catch (EndpointNotFoundException ex) { HandleEndpointException(ex); }
                    }
                }
            }
        }

        #endregion

        private void EditPasteMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void EditCutMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void EditCopyMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void EditRenameMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (BListView.ListView.SelectedIndices.Count > 0)
                {
                    ListView list = BListView.ListView;
                    ListViewItem item = list.Items[list.SelectedIndices[0]];
                    DirectoryInfo info = item.Tag as DirectoryInfo;

                    if (info != null)
                    {
                        using (FolderRenameForm form = new FolderRenameForm(info.Name))
                        {
                            if (form.ShowDialog().Equals(DialogResult.OK))
                            {
                                string oldname = info.FullName;
                                string newname = Path.Combine(Path.GetDirectoryName(oldname), form.NewName);

                                Service.MoveTo(oldname, newname);
                            }
                        }
                    }

                    RefreshMenuItem_Click(this, EventArgs.Empty);
                }
            }
            catch (Microsoft.Tools.ServiceModel.CFFaultException ex) { HandleCFException(ex.FaultMessage); }
            catch (EndpointNotFoundException ex) { HandleEndpointException(ex); }
        }

        private void EditNewFolderItem_Click(object sender, EventArgs e)
        {
            using (NewFolderForm form = new NewFolderForm())
            {
                if (form.ShowDialog().Equals(DialogResult.OK))
                {
                    DirectoryInfo info = Service.GetDirectoryInfo(mPath);

                    try
                    {
                        DirectoryInfo newdir = Service.CreateSubdirectory(info.FullName, form.FolderName, (int)form.Attributes);

                        BListView.AddListViewItemDirectory(newdir);
                    }
                    catch (Microsoft.Tools.ServiceModel.CFFaultException ex) { HandleCFException(ex.FaultMessage); }
                    catch (EndpointNotFoundException ex) { HandleEndpointException(ex); }
                }
            }
        }

        private void SortTypeMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void SortSizeMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void SortDateMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void SortNameMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void AboutMenuItem_Click(object sender, EventArgs e)
        {
            using (AboutForm form = new AboutForm()) form.ShowDialog();
        }

        private void BListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (BListView.ListView.SelectedIndices.Count > 0 && mLevel > 2)
                EditMenuItem.Enabled = true;
            else
                EditMenuItem.Enabled = false;
        }

        private void SearchMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {

        }
    }
}