﻿using System;
using System.Windows.Forms;
using System.Linq;

namespace Sycamores.MobileShare.Client
{
    /// <summary>Base form class for numbered menu forms.</summary>
    public partial class MenuForm : Form
    {
        /// <summary>Default constructor.</summary>
        public MenuForm() { InitializeComponent(); }

        /// <summary>Create form with or without a 'New' line item.</summary>
        /// <param name="useNew">Create a 'New' line item.</param>
        public MenuForm(bool useNew) : this() { UseNew = useNew; }

        private bool mUseNew = true;

        /// <summary>Create form with a 'New' line item.</summary>
        public bool UseNew
        {
            get { return mUseNew; }
            set
            {
                if (!mUseNew.Equals(value))
                {
                    mUseNew = value;

                    var query = from TreeNode node in MenuTree.Nodes
                                where node.Text.Equals(Properties.Resources.MenuNewText)
                                select node;

                    if (mUseNew)
                    {
                        if (query.Count() <= 0) MenuTree.Nodes.Add(Properties.Resources.MenuNewText);
                    }
                    else
                    {
                        while (query.Count() > 0) MenuTree.Nodes.Remove(query.First());
                    }
                }

                MenuMenuItem.Enabled = value;
                if (!value) MenuMenuItem.Text = string.Empty;
            }
        }

        /// <summary>Done.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected void DoneMenuItem_Click(object sender, EventArgs e) { this.DialogResult = DialogResult.OK; }

        /// <summary>Add item.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected virtual void AddMenuItem_Click(object sender, EventArgs e) { }

        /// <summary>Edit item.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected virtual void EditMenuItem_Click(object sender, EventArgs e) { }

        /// <summary>Delete item.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected virtual void DeleteMenuItem_Click(object sender, EventArgs e) { }

        /// <summary>Item selected.</summary>
        protected virtual void ItemSelected() { }

        #region Menu Operation

        /// <summary>Add item.</summary>
        /// <param name="text">Text value to display in line item.</param>
        protected void Add(string text)
        {
            string newnumber = (MenuTree.Nodes.Count + 1).ToString();

            MenuTree.Nodes.Add(string.Concat(newnumber, "  ", text));
        }

        /// <summary>Clear line items.</summary>
        /// <remarks>Replace 'New' line item if UseNew flag is set.</remarks>
        protected void Clear()
        {
            MenuTree.Nodes.Clear();

            if (UseNew) MenuTree.Nodes.Add(Properties.Resources.MenuNewText);
        }

        /// <summary>Index of the selected line item.</summary>
        protected int SelectedIndex
        {
            get { return MenuTree.Nodes.IndexOf(MenuTree.SelectedNode); }
            set { MenuTree.SelectedNode = MenuTree.Nodes[value]; }
        }

        /// <summary>Numeric value of the selected line item.</summary>
        /// <remarks>Numeric values are 1-based.</remarks>
        public string SelectedValue { get { return MenuTree.SelectedNode.Text.Substring(3); } }

        /// <summary>Remove the presently selected line item.</summary>
        protected void RemoveSelection() { MenuTree.Nodes.Remove(MenuTree.SelectedNode); }

        private void MenuTree_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar.Equals('\r'))
            {
                ItemSelected();
            }
            else
            {
                try
                {
                    int value = Convert.ToInt32(e.KeyChar.ToString());

                    if (value > 0 && value <= MenuTree.Nodes.Count)
                    {
                        MenuTree.SelectedNode = MenuTree.Nodes[value - 1];
                        ItemSelected();
                    }
                }
                catch { /* Ignore keys that do not correspond to a line item. */ }
            }
        }

        #endregion
    }
}