﻿using System;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Schema;
using System.Xml.Xsl;
using System.Text;
using System.Reflection;

namespace Sycamores.XFrameworkCF
{
    /// <summary>Validating XML file.</summary>
    /// <remarks>Opens or creates a new XML file validating any existing data according to an XML schema file.</remarks>
    public class ValidatingXmlFile
    {
        private string mFilename;
        private XDocument mDocument;

        private object Locker = new object();

        /// <summary>XML filename.</summary>
        public string Filename
        {
            get { return mFilename; }
            protected set { mFilename = value; }
        }

        /// <summary>Document member.</summary>
        public XDocument Document { get { return mDocument; } }

        /// <summary>Default constructor.</summary>
        /// <param name="nameSpace">Default namespace of the XML file.</param>
        /// <param name="rootTag">File root tag.</param>
        /// <param name="filename">XML file path.</param>
        /// <param name="schemaFilename">XML schema file path.</param>
        /// <remarks>Opens and validates an XML file. If the file does not exist, a new file is created.</remarks>
        public ValidatingXmlFile(string filename, XmlSchema schema, XDocument defaultDoc)
        {
            if (File.Exists(filename))
            {
                using (Stream input = File.OpenRead(filename))
                    Initialize(filename, input, schema); 
            }
            else
            {
                mDocument = defaultDoc;

                mDocument.Save(filename);
                //mRoot.Changed += new EventHandler<XObjectChangeEventArgs>(Root_Changed);
            }
        }

        protected ValidatingXmlFile(string filename, Stream input, XmlSchema schema)
        {
            Initialize(filename, input, schema);
        }

        private void Initialize(string filename, Stream input, XmlSchema schema)
        {
            lock (Locker)
            {
                XmlReader reader = null;

                try
                {
                    XmlSchemaSet schemas = new XmlSchemaSet();
                    schemas.Add(schema);

                    mFilename = filename;

                    XmlReader versionreader = XmlReader.Create(input);
                    mDocument = XDocument.Load(versionreader);
                    versionreader.Close();
                    UpdateFile();

                    XmlReaderSettings settings = new XmlReaderSettings();
                    settings.ValidationType = ValidationType.Schema;
                    settings.Schemas = schemas;
                    settings.ValidationEventHandler += new ValidationEventHandler(ValidationHandler);

                    input.Position = 0;
                    reader = XmlReader.Create(input, settings);
                    mDocument = XDocument.Load(reader);
                }
                catch { throw; }
                finally { if (reader != null) reader.Close(); }

                //mRoot.Changed += new EventHandler<XObjectChangeEventArgs>(Root_Changed);
            }
        }

        //public virtual void Root_Changed(object sender, XObjectChangeEventArgs e)
        //{
        //    throw new NotImplementedException();
        //}

        /// <summary>Represents the callback method that will handle XML schema validation events and the System.Xml.Schema.ValidationEventArgs.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An System.Xml.Schema.ValidationEventArgs containing the event data.</param>
        protected static void ValidationHandler(object sender, ValidationEventArgs e)
        {
            throw new XmlSchemaValidationException(e.Message, e.Exception);
        }

        /// <summary>Update data changes to the opened data file.</summary>
        public virtual void Save()
        {
            Save(mFilename);
        }

        public virtual void Save(string filename)
        {
            mDocument.Save(filename, SaveOptions.None);
        }

        /// <summary>Retrieve embedded resourced XML schema document (XSD).</summary>
        /// <param name="resourcename">Resource name of the embedded file.</param>
        /// <remarks>Retrieves an embedded XSD file as an <see cref="System.Xml.Schema.XmlSchema">XmlSchema</see> file. 
        /// at the <paramref cref="filename">fileName.</see></remarks>
        public static XmlSchema GetEmbeddedResourceSchema(string resourcename)
        {
            XmlSchema schema = null;

            using (Stream stream = GetEmbeddedResourceFile(Assembly.GetCallingAssembly(), resourcename))
                schema = XmlSchema.Read(stream, ValidationHandler);

            return schema;
        }

        /// <summary>Retrieve embedded resourced XML document.</summary>
        /// <param name="resourcename">Resource name of the embedded file.</param>
        /// <remarks>Retrieves an embedded XML file as an <see cref="System.Xml.Linq.XDocument">XDocument</see> file. 
        /// at the <paramref cref="filename">fileName.</see></remarks>
        public static XDocument GetEmbeddedResourceXDoc(string resourcename)
        {
            XDocument doc = null;
            XmlReader reader = null;

            try
            {
                using (Stream stream = GetEmbeddedResourceFile(Assembly.GetCallingAssembly(), resourcename))
                {
                    XmlReaderSettings settings = new XmlReaderSettings();
                    settings.ConformanceLevel = ConformanceLevel.Document;

                    reader = XmlReader.Create(stream, settings);
                    doc = XDocument.Load(reader);
                }
            }
            catch { throw; }
            finally { reader.Close(); }

            return doc;
        }

        /// <summary>Write embedded resourced XML document.</summary>
        /// <param name="resourcename">Resource name of the embedded file.</param>
        /// <param name="fileName">Filename to save the XML file.</param>
        /// <remarks>Saves an embedded XML file to an XML file at the <paramref cref="filename">fileName.</see></remarks>
        public static void WriteEmbeddedResourceXml(string resourcename, string fileName)
        {
            string targetdir = Path.GetDirectoryName(fileName);

            if (!Directory.Exists(targetdir)) Directory.CreateDirectory(targetdir);

            using (Stream stream = GetEmbeddedResourceFile(Assembly.GetCallingAssembly(), resourcename))
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(stream);
                doc.Save(fileName);
            }
        }

        /// <summary>Get embedded resource file stream.</summary>
        /// <param name="resourcename">Resource name of the embedded file.</param>
        /// <returns>Stream containing the embedded resource file content.</returns>
        public static Stream GetEmbeddedResourceFile(string resourcename)
        {
            return GetEmbeddedResourceFile(Assembly.GetCallingAssembly(), resourcename);
        }

        /// <summary>Get embedded resource file stream.</summary>
        /// <param name="resourcename">Resource name of the embedded file.</param>
        /// <returns>Stream containing the embedded resource file content.</returns>
        private static Stream GetEmbeddedResourceFile(System.Reflection.Assembly thisassembly, string resourcename)
        {
            return thisassembly.GetManifestResourceStream(resourcename);
        }

        /// <summary>Get embedded resource file stream.</summary>
        /// <param name="resourcename">Resource name of the embedded file.</param>
        /// <returns>Stream containing the embedded resource file content.</returns>
        public static string GetEmbeddedResourceFileXmlText(string resourcename)
        {
            string content = null;

            using (Stream stream = GetEmbeddedResourceFile(Assembly.GetCallingAssembly(),resourcename))
            {
                XmlReader reader = XmlReader.Create(stream);

                content = reader.ReadOuterXml();
            }

            return content;
        }

        /// <summary>Update file.</summary>
        /// <remarks>Overriding classes can redefine this method to do whatever they need in order to update an XML file already
        /// existing on the target machine to a new XML schema according to version information in the file.</remarks>
        public virtual void UpdateFile() { }

        /// <summary>Update file using an XSL transform.</summary>
        /// <param name="xml">The target XML filename. This will also be overwritten with the transformed output.</param>
        /// <param name="xslt">The filename of the XSL transform file.</param>
        /// <remarks>This uses the built-in MSXML2 imported through a type library.</remarks>
        public static void UpdateFile(string xml, string xslt)
        {
            MSXML2.DOMDocument doc = new MSXML2.DOMDocumentClass();
            doc.load(xml);

            MSXML2.DOMDocument xsl = new MSXML2.DOMDocumentClass();
            xsl.loadXML(xslt);

            XDocument xdoc = XDocument.Parse(doc.transformNode(xsl));

            xdoc.Save(xml);
        }
    }
}