﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Xml.Linq;

using Sycamores.XFramework;

namespace Sycamores.MobileShare.Communications
{
    /// <summary>Application Data Element.</summary>
    public class ApplicationDataElement : ComplexType
    {
        private const string cTag = "applicationData";
        private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Communications/applicationData";

        /// <summary>Default constructor.</summary>
        /// <param name="element">Element object.</param>
        public ApplicationDataElement(XElement element) : base(element, cNamespace) { }

        /// <summary>String value of this elements tag in the XML schema.</summary>
        public override string Tag { get { return cTag; } }

        #region CookieLifeSpan

        private const string cCookieLifeSpan = "cookieLifeSpan";

        /// <summary>Hide system file flag is changing.</summary>
        public event EventHandler CookieLifeSpanChanging;
        /// <summary>Hide system file flag has changed.</summary>
        public event EventHandler CookieLifeSpanChanged;

        /// <summary>Hide system files.</summary>
        /// <remarks>Configuration flag which determines whether system files will be shown in the list view.</remarks>
        public TimeSpan CookieLifeSpan
        {
            get { return GetSimpleRequiredElementTimeSpan(cCookieLifeSpan); }
            set { SetSimpleRequiredElement<TimeSpan>(cCookieLifeSpan, value, CookieLifeSpanChanging, CookieLifeSpanChanged); }
        }

        #endregion

        #region Account

        private ApplicationData.AccountElementCollection mAccounts;

        public ApplicationData.AccountElementCollection Accounts
        {
            get
            {
                if (mAccounts == null)
                {
                    mAccounts = new ApplicationData.AccountElementCollection(Element);

                    mAccounts.ItemRemoving += new EventHandler<Sycamores.MobileShare.Communications.ApplicationData.AccountsEventArgs>(mAccounts_ItemRemoving);
                    mAccounts.ItemRemoved += new EventHandler<Sycamores.MobileShare.Communications.ApplicationData.AccountsEventArgs>(mAccounts_ItemRemoved);
                }

                return mAccounts;
            }
        }

        void mAccounts_ItemRemoved(object sender, Sycamores.MobileShare.Communications.ApplicationData.AccountsEventArgs e) { }

        void mAccounts_ItemRemoving(object sender, Sycamores.MobileShare.Communications.ApplicationData.AccountsEventArgs e) { }

        #endregion
    }

    namespace ApplicationData
    {
        public class AccountElement : ComplexType
        {
            private const string cTag = "account";
            private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Communications/applicationData";

            public AccountElement(XElement element) : base(element, cNamespace) { }

            #region Id

            private const string cId = "id";
            private const string dId = null;

            public event EventHandler IdChanging;
            public event EventHandler IdChanged;

            public string Id
            {
                get { return GetRequiredAttributeString(cId, dId); }
                set { SetRequiredAttribute<string>(cId, value, IdChanging, IdChanged); }
            }

            #endregion

            #region Date

            private const string cDate = "date";
            private const string dDate = null;

            public event EventHandler DateChanging;
            public event EventHandler DateChanged;

            public DateTime  Date
            {
                get { return GetRequiredAttributeDateTime(cDate, dDate); }
                set { SetRequiredAttribute<DateTime>(cDate, value, DateChanging, DateChanged); }
            }

            #endregion

            #region Username

            private const string cUsername = "username";

            public event EventHandler UsernameChanging;
            public event EventHandler UsernameChanged;

            public string Username
            {
                get { return GetSimpleRequiredElementString(cUsername, string.Empty); }
                set { SetSimpleRequiredElement<string>(cUsername, value, UsernameChanging, UsernameChanged); }
            }

            #endregion

            #region Domain

            private const string cDomain = "domain";

            public event EventHandler DomainChanging;
            public event EventHandler DomainChanged;

            public string Domain
            {
                get { return GetSimpleRequiredElementString(cDomain, string.Empty); }
                set { SetSimpleRequiredElement<string>(cDomain, value, DomainChanging, DomainChanged); }
            }

            #endregion

            #region Password

            private const string cPassword = "password";

            public event EventHandler PasswordChanging;
            public event EventHandler PasswordChanged;

            public string Password
            {
                get { return GetSimpleRequiredElementString(cPassword, string.Empty); }
                set { SetSimpleRequiredElement<string>(cPassword, value, PasswordChanging, PasswordChanged); }
            }

            #endregion

            public override string Tag
            {
                get { return cTag; }
            }
        }

        public class AccountElementCollection : KeyedComplexTypeCollection<string, AccountElement, AccountsEventArgs>
        {
            private const string cTag = "account";
            private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Communications/applicationData";
            private const string cId = "id";

            protected internal AccountElementCollection(XElement element)
                : base(cNamespace, element)
            {
                lock (Locker)
                {
                    foreach (var item in element.Elements(GetElementName(cTag)))
                    {
                        Add(item.Attribute(cId).Value, new AccountElement(item));
                    }
                }
            }

            public AccountElement GetNewAccount()
            {
                return new AccountElement(new XElement(GetElementName(cTag)));
            }

            public void Add(AccountElement value)
            {
                lock (Locker)
                {
                    var query = from account in Dictionary.Values
                                where account.Domain.ToUpperInvariant().Equals(value.Domain.ToUpperInvariant())
                                && account.Username.ToUpperInvariant().Equals(value.Username.ToUpperInvariant())
                                select account;

                    if (query.Count() > 0) throw new InvalidOperationException("A similar account already exists.");

                    Dictionary.Add(value.Id, value);
                    Add(GetElementName(cTag), value);
                    value.IdChanged += new EventHandler(value_IdChanged);
                    value.IdChanging += new EventHandler(value_IdChanging);
                }
            }

            public new event EventHandler<AccountsEventArgs> ItemRemoving;
            public new event EventHandler<AccountsEventArgs> ItemRemoved;

            public new bool Remove(string key)
            {
                lock (Locker)
                {
                    if (ItemRemoving != null) ItemRemoving(this, new AccountsEventArgs(this[key]));

                    this[key].IdChanged -= new EventHandler(value_IdChanged);
                    this[key].IdChanging -= new EventHandler(value_IdChanging);

                    var query = from account in Element.Elements(GetElementName(cTag))
                                where account.Attributes(GetAttributeName("id")).First().Value.Equals(key)
                                select account;

                    foreach (XElement item in query) item.Remove();

                    if (ItemRemoved != null) ItemRemoved(this, new AccountsEventArgs(this[key]));
                }

                return Dictionary.Remove(key);
            }

            void value_IdChanging(object sender, EventArgs e)
            {
                throw new NotImplementedException();
            }

            void value_IdChanged(object sender, EventArgs e)
            {
                throw new NotImplementedException();
            }

            public override string Tag
            {
                get { return cTag; }
            }

            public override void Add(XName tag, AccountElement value)
            {
                if (Element.Elements(cTag).Count() > 0)
                    Element.Elements(cTag).Last().AddAfterSelf(value.Element);
                else
                    Element.Elements(GetElementName("cookieLifeSpan")).First().AddAfterSelf(value.Element);
            }
        }

        public class AccountsEventArgs : EventArgs
        {
            private AccountElement mElement;

            public AccountsEventArgs(AccountElement element)
            {
                mElement = element;
            }

            public AccountElement Element { get { return mElement; } }
        }
    }
}