﻿using System;
using System.Runtime.InteropServices;

namespace Sycamores.PlatformInvoke.Networking
{
    public struct ServerInfo
    {
        public int PlatformId;
        public string Name;
    }

    /// <summary>
    /// The possible flag values for Server Type (see lmserver.h).
    /// </summary>
    [Flags]
    public enum ServerType : long
    {
        /// <summary>
        /// Opposite of All.  No servers will be returned.
        /// </summary>
        None = 0x00000000,
        /// <summary>
        /// All workstations
        /// </summary>
        Workstation = 0x00000001,
        /// <summary>
        /// All servers
        /// </summary>
        Server = 0x00000002,
        /// <summary>
        /// Any server running with Microsoft SQL Server
        /// </summary>
        SQLServer = 0x00000004,
        /// <summary>
        /// Primary domain controller
        /// </summary>
        DomainController = 0x00000008,
        /// <summary>
        /// Backup domain controller
        /// </summary>
        DomainBackupController = 0x00000010,
        /// <summary>
        /// Server running the Timesource service
        /// </summary>
        TimeSource = 0x00000020,
        /// <summary>
        /// Apple File Protocol servers
        /// </summary>
        AFP = 0x00000040,
        /// <summary>
        /// Novell servers
        /// </summary>
        Novell = 0x00000080,
        /// <summary>
        /// LAN Manager 2.x domain member
        /// </summary>
        DomainMember = 0x00000100,
        /// <summary>
        /// Server sharing print queue
        /// </summary>
        PrintQueue = 0x00000200,
        /// <summary>
        /// Server running dial-in service
        /// </summary>
        Dialin = 0x00000400,
        /// <summary>
        /// Xenix server
        /// </summary>
        Xenix = 0x00000800,
        /// <summary>
        /// Unix servers?
        /// </summary>
        Unix = Xenix,
        /// <summary>
        /// Windows NT workstation or server
        /// </summary>
        NT = 0x00001000,
        /// <summary>
        /// Server running Windows for Workgroups
        /// </summary>
        WFW = 0x00002000,
        /// <summary>
        /// Microsoft File and Print for NetWare
        /// </summary>
        MFPN = 0x00004000,
        /// <summary>
        /// Server that is not a domain controller
        /// </summary>
        NTServer = 0x00008000,
        /// <summary>
        /// Server that can run the browser service
        /// </summary>
        PotentialBrowser = 0x00010000,
        /// <summary>
        /// Server running a browser service as backup
        /// </summary>
        BackupBrowser = 0x00020000,
        /// <summary>
        /// Server running the master browser service
        /// </summary>
        MasterBrowser = 0x00040000,
        /// <summary>
        /// Server running the domain master browser
        /// </summary>
        DomainMaster = 0x00080000,
        /// <summary>
        /// Not documented on MSDN? Help Microsoft!
        /// </summary>
        OSF = 0x00100000,
        /// <summary>
        /// Running VMS
        /// </summary>
        VMS = 0x00200000,
        /// <summary>
        /// Windows 95 or later
        /// </summary>
        Windows = 0x00400000,
        /// <summary>
        /// Distributed File System??
        /// </summary>
        DFS = 0x00800000,
        /// <summary>
        /// Not documented on MSDN? Help Microsoft!
        /// </summary>
        ClusterNT = 0x01000000,
        /// <summary>
        /// Terminal Server
        /// </summary>
        TerminalServer = 0x02000000,
        /// <summary>
        /// Not documented on MSDN? Help Microsoft!
        /// </summary>
        DCE = 0x10000000,
        /// <summary>
        /// Not documented on MSDN? Help Microsoft!
        /// </summary>
        AlternateXPort = 0x20000000,
        /// <summary>
        /// Servers maintained by the browser
        /// </summary>
        ListOnly = 0x40000000,
        /// <summary>
        /// List Domains
        /// </summary>
        DomainEnum = 0x80000000,
        /// <summary>
        /// All servers
        /// </summary>
        All = 0xFFFFFFFF
    }

    public class ManagementAccess
    {
        public static ServerInfo[] GetServers(ServerType serverType, string domainName)
        {
            const uint level = 100;
            uint entriesread = 0;
            uint totalentries = 0;
            uint result;

            IntPtr serverinfo = IntPtr.Zero;

            try
            {
                result = Management.NetServerEnum(IntPtr.Zero,
                    level,
                    ref serverinfo,
                    Management.MAX_PREFERRED_LENGTH,
                    ref entriesread,
                    ref totalentries,
                    (uint)serverType,
                    domainName,
                    IntPtr.Zero);

                switch (result)
                {
                    case 0:
                        Type sharetype = typeof(Management.SERVER_INFO_100);
                        int offset = Marshal.SizeOf(sharetype);
                        ServerInfo[] servers = new ServerInfo[entriesread];

                        for (int i = 0, item = serverinfo.ToInt32(); i < entriesread; i++, item += offset)
                        {
                            IntPtr pitem = new IntPtr(item);
                            Management.SERVER_INFO_100 si = (Management.SERVER_INFO_100)Marshal.PtrToStructure(pitem, sharetype);
                            servers[i].Name = Marshal.PtrToStringAuto(si.lpszServerName);
                            servers[i].PlatformId = si.dwPlatformID;
                        }


                        return servers;
                    default:
                        throw new InvalidOperationException();
                }
            }
            catch { throw; }
            finally
            {
                if (!serverinfo.Equals(IntPtr.Zero))
                {
                    Management.NetApiBufferFree(serverinfo);
                    serverinfo = IntPtr.Zero;
                }
            }
        }
    }

    internal static class Management
    {
        internal const uint MAX_PREFERRED_LENGTH = 0xFFFFFFFF;

        internal enum PlatformIdCodes : int
        {
            PLATFORM_ID_DOS = 300,
            PLATFORM_ID_OS2 = 400,
            PLATFORM_ID_NT = 500,
            PLATFORM_ID_OSF = 600,
            PLATFORM_ID_VMS = 700,
        }

        [StructLayoutAttribute(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        internal struct SERVER_INFO_100
        {
            public int dwPlatformID;
            public IntPtr lpszServerName;
        }

        [StructLayoutAttribute(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        internal struct SERVER_INFO_101
        {
            public PlatformIdCodes dwPlatformID;
            public IntPtr lpszServerName;
            public int dwVersionMajor;
            public int dwVersionMinor;
            public int dwType;
            public IntPtr lpszComment;
        }

        [DllImport("netapi32.dll", EntryPoint = "NetApiBufferFree")]
        internal static extern int NetApiBufferFree(IntPtr bufptr);

        [DllImport("netapi32.dll", EntryPoint = "NetServerEnum")]
        internal static extern uint NetServerEnum(
            IntPtr ServerName,
            uint level,
            ref IntPtr siPtr,
            uint prefmaxlen,
            ref uint entriesread,
            ref uint totalentries,
            uint servertype,
            [MarshalAs(UnmanagedType.LPWStr)] string domain,
            IntPtr resumeHandle);

        /// <summary></summary>
        /// <remarks>Windows NT/2000/XP Only</remarks>
        [DllImport("netapi32.dll", EntryPoint = "NetServerGetInfo")]
        internal static extern uint NetServerGetInfo(
            [MarshalAs(UnmanagedType.LPWStr)] string ServerName,
            int level,
            ref IntPtr buffPtr);
    }
}